@@ -1,6 +1,6 @@
 /* OsmoHLR Control Interface implementation */
 
-/* (C) 2017 sysmocom s.f.m.c. GmbH <info@sysmocom.de>
+/* (C) 2017-2023 sysmocom s.f.m.c. GmbH <info@sysmocom.de>
  * All Rights Reserved
  *
  * Author: Max Suraev <msuraev@sysmocom.de>
@@ -39,7 +39,7 @@
 #define SEL_BY_ID SEL_BY "id-"
 
 extern bool auth_algo_parse(const char *alg_str, enum osmo_auth_algo *algo,
-			    int *minlen, int *maxlen);
+			    int *minlen, int *maxlen, int *minlen_opc, int *maxlen_opc);
 
 #define hexdump_buf(buf) osmo_hexdump_nospc((void*)buf, sizeof(buf))
 
@@ -166,7 +166,7 @@ static void print_subscr_info(struct ctrl_cmd *cmd,
 		);
 }
 
-static void print_subscr_info_aud2g(struct ctrl_cmd *cmd, struct osmo_sub_auth_data *aud)
+static void print_subscr_info_aud2g(struct ctrl_cmd *cmd, struct osmo_sub_auth_data2 *aud)
 {
 	if (aud->algo == OSMO_AUTH_ALG_NONE)
 		return;
@@ -178,7 +178,7 @@ static void print_subscr_info_aud2g(struct ctrl_cmd *cmd, struct osmo_sub_auth_d
 		hexdump_buf(aud->u.gsm.ki));
 }
 
-static void print_subscr_info_aud3g(struct ctrl_cmd *cmd, struct osmo_sub_auth_data *aud)
+static void print_subscr_info_aud3g(struct ctrl_cmd *cmd, struct osmo_sub_auth_data2 *aud)
 {
 	if (aud->algo == OSMO_AUTH_ALG_NONE)
 		return;
@@ -187,7 +187,7 @@ static void print_subscr_info_aud3g(struct ctrl_cmd *cmd, struct osmo_sub_auth_d
 		"\naud3g.k\t%s"
 		,
 		osmo_auth_alg_name(aud->algo),
-		hexdump_buf(aud->u.umts.k));
+		osmo_hexdump_nospc(aud->u.umts.k, aud->u.umts.k_len));
 	/* hexdump uses a static string buffer, hence only one hexdump per
 	 * printf(). */
 	ctrl_cmd_reply_printf(cmd,
@@ -196,7 +196,7 @@ static void print_subscr_info_aud3g(struct ctrl_cmd *cmd, struct osmo_sub_auth_d
 		"\naud3g.sqn\t%" PRIu64
 		,
 		aud->u.umts.opc_is_op? "op" : "opc",
-		hexdump_buf(aud->u.umts.opc),
+		osmo_hexdump_nospc(aud->u.umts.opc, aud->u.umts.opc_len),
 		aud->u.umts.ind_bitlen,
 		aud->u.umts.sqn);
 }
@@ -291,8 +291,8 @@ CTRL_CMD_DEFINE_RO(subscr_info_aud, "info-aud");
 static int get_subscr_info_aud(struct ctrl_cmd *cmd, void *data)
 {
 	const char *imsi;
-	struct osmo_sub_auth_data aud2g;
-	struct osmo_sub_auth_data aud3g;
+	struct osmo_sub_auth_data2 aud2g;
+	struct osmo_sub_auth_data2 aud3g;
 	struct hlr *hlr = data;
 	const char *by_selector = cmd->node;
 	int rc;
@@ -327,8 +327,8 @@ CTRL_CMD_DEFINE_RO(subscr_info_all, "info-all");
 static int get_subscr_info_all(struct ctrl_cmd *cmd, void *data)
 {
 	struct hlr_subscriber subscr;
-	struct osmo_sub_auth_data aud2g;
-	struct osmo_sub_auth_data aud3g;
+	struct osmo_sub_auth_data2 aud2g;
+	struct osmo_sub_auth_data2 aud3g;
 	struct hlr *hlr = data;
 	const char *by_selector = cmd->node;
 	int rc;
@@ -426,6 +426,20 @@ static int set_subscr_cs_enabled(struct ctrl_cmd *cmd, void *data)
 	return set_subscr_cs_ps_enabled(cmd, data, false);
 }
 
+CTRL_CMD_DEFINE_RO(subscr_imsi, "imsi");
+static int get_subscr_imsi(struct ctrl_cmd *cmd, void *data)
+{
+	struct hlr_subscriber subscr;
+	struct hlr *hlr = data;
+	const char *by_selector = cmd->node;
+
+	if (!get_subscriber(hlr->dbc, by_selector, &subscr, cmd))
+		return CTRL_CMD_ERROR;
+
+	cmd->reply = talloc_strdup(cmd, subscr.imsi);
+	return CTRL_CMD_REPLY;
+}
+
 CTRL_CMD_DEFINE(subscr_msisdn, "msisdn");
 static int verify_subscr_msisdn(struct ctrl_cmd *cmd, const char *value, void *data)
 {
@@ -492,8 +506,8 @@ static int get_subscr_aud2g(struct ctrl_cmd *cmd, void *data)
 	struct hlr_subscriber subscr;
 	struct hlr *hlr = data;
 	const char *by_selector = cmd->node;
-	struct osmo_sub_auth_data aud2g;
-	struct osmo_sub_auth_data aud3g_unused;
+	struct osmo_sub_auth_data2 aud2g;
+	struct osmo_sub_auth_data2 aud3g_unused;
 	int rc;
 
 	if (!get_subscriber(hlr->dbc, by_selector, &subscr, cmd))
@@ -550,7 +564,7 @@ static int set_subscr_aud2g(struct ctrl_cmd *cmd, void *data)
 	}
 	if (strcmp(tok, "none") == 0) {
 		aud2g.algo = OSMO_AUTH_ALG_NONE;
-	} else if (!auth_algo_parse(tok, &aud2g.algo, &minlen, &maxlen)) {
+	} else if (!auth_algo_parse(tok, &aud2g.algo, &minlen, &maxlen, NULL, NULL)) {
 		cmd->reply = "Unknown auth algorithm.";
 		return CTRL_CMD_ERROR;
 	}
@@ -592,8 +606,8 @@ static int get_subscr_aud3g(struct ctrl_cmd *cmd, void *data)
 	struct hlr_subscriber subscr;
 	struct hlr *hlr = data;
 	const char *by_selector = cmd->node;
-	struct osmo_sub_auth_data aud2g_unused;
-	struct osmo_sub_auth_data aud3g;
+	struct osmo_sub_auth_data2 aud2g_unused;
+	struct osmo_sub_auth_data2 aud3g;
 	int rc;
 
 	if (!get_subscriber(hlr->dbc, by_selector, &subscr, cmd))
@@ -618,9 +632,9 @@ static int get_subscr_aud3g(struct ctrl_cmd *cmd, void *data)
 	}
 
 	cmd->reply = talloc_asprintf(cmd, "%s,%s,%s,%s,%u", osmo_auth_alg_name(aud3g.algo),
-				     osmo_hexdump_nospc_c(cmd, aud3g.u.umts.k, sizeof(aud3g.u.umts.k)),
+				     osmo_hexdump_nospc_c(cmd, aud3g.u.umts.k, aud3g.u.umts.k_len),
 				     aud3g.u.umts.opc_is_op ? "OP" : "OPC",
-				     osmo_hexdump_nospc_c(cmd, aud3g.u.umts.opc, sizeof(aud3g.u.umts.opc)),
+				     osmo_hexdump_nospc_c(cmd, aud3g.u.umts.opc, aud3g.u.umts.opc_len),
 				     aud3g.u.umts.ind_bitlen);
 	return CTRL_CMD_REPLY;
 }
@@ -630,8 +644,8 @@ static int set_subscr_aud3g(struct ctrl_cmd *cmd, void *data)
 	struct hlr *hlr = data;
 	const char *by_selector = cmd->node;
 	char *tmp = NULL, *tok, *saveptr;
-	int minlen = 0;
-	int maxlen = 0;
+	int minlen = 0, minlen_opc = 0;
+	int maxlen = 0, maxlen_opc = 0;
 	struct sub_auth_data_str aud3g = {
 		.type = OSMO_AUTH_TYPE_UMTS,
 		.u.umts = {
@@ -657,7 +671,7 @@ static int set_subscr_aud3g(struct ctrl_cmd *cmd, void *data)
 	}
 	if (strcmp(tok, "none") == 0) {
 		aud3g.algo = OSMO_AUTH_ALG_NONE;
-	} else if (!auth_algo_parse(tok, &aud3g.algo, &minlen, &maxlen)) {
+	} else if (!auth_algo_parse(tok, &aud3g.algo, &minlen, &maxlen, &minlen_opc, &maxlen_opc)) {
 		cmd->reply = "Unknown auth algorithm.";
 		return CTRL_CMD_ERROR;
 	}
@@ -699,7 +713,7 @@ static int set_subscr_aud3g(struct ctrl_cmd *cmd, void *data)
 		}
 
 		aud3g.u.umts.opc = tok;
-		if (!osmo_is_hexstr(aud3g.u.umts.opc, MILENAGE_KEY_LEN * 2, MILENAGE_KEY_LEN * 2, true)) {
+		if (!osmo_is_hexstr(aud3g.u.umts.opc, minlen_opc * 2, maxlen_opc * 2, true)) {
 			cmd->reply = talloc_asprintf(cmd, "Invalid OP/OPC.");
 			return CTRL_CMD_ERROR;
 		}
@@ -749,7 +763,7 @@ static int hlr_ctrl_node_lookup(void *data, vector vline, int *node_type,
 	return 1;
 }
 
-static int hlr_ctrl_cmds_install()
+static int hlr_ctrl_cmds_install(void)
 {
 	int rc = 0;
 
@@ -761,6 +775,7 @@ static int hlr_ctrl_cmds_install()
 	rc |= ctrl_cmd_install(CTRL_NODE_SUBSCR_BY, &cmd_subscr_info_all);
 	rc |= ctrl_cmd_install(CTRL_NODE_SUBSCR_BY, &cmd_subscr_ps_enabled);
 	rc |= ctrl_cmd_install(CTRL_NODE_SUBSCR_BY, &cmd_subscr_cs_enabled);
+	rc |= ctrl_cmd_install(CTRL_NODE_SUBSCR_BY, &cmd_subscr_imsi);
 	rc |= ctrl_cmd_install(CTRL_NODE_SUBSCR_BY, &cmd_subscr_msisdn);
 	rc |= ctrl_cmd_install(CTRL_NODE_SUBSCR_BY, &cmd_subscr_aud2g);
 	rc |= ctrl_cmd_install(CTRL_NODE_SUBSCR_BY, &cmd_subscr_aud3g);
@@ -771,11 +786,8 @@ static int hlr_ctrl_cmds_install()
 struct ctrl_handle *hlr_controlif_setup(struct hlr *hlr)
 {
 	int rc;
-	struct ctrl_handle *hdl = ctrl_interface_setup_dynip2(hlr,
-							      hlr->ctrl_bind_addr,
-							      OSMO_CTRL_PORT_HLR,
-							      hlr_ctrl_node_lookup,
-							      _LAST_CTRL_NODE_HLR);
+	struct ctrl_handle *hdl = ctrl_interface_setup2(hlr, OSMO_CTRL_PORT_HLR, hlr_ctrl_node_lookup,
+							_LAST_CTRL_NODE_HLR);
 	if (!hdl)
 		return NULL;
 
